/*
Kooboo is a content management system based on ASP.NET MVC framework. Copyright 2009 Yardi Technology Limited.

This program is free software: you can redistribute it and/or modify it under the terms of the
GNU General Public License version 3 as published by the Free Software Foundation.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program.
If not, see http://www.kooboo.com/gpl3/.
*/
using System;
using System.Collections.Generic;
using System.Linq;
using System.Web.Mvc;
using Everest.Library.Extjs;
using Everest.Library.Extjs.Tree;
using Everest.Library.Json;
using Everest.Library.Mvc;
namespace ExtjsWeb.Controllers
{
    public class User
    {
        public string Name { get; set; }
        public string CardID { get; set; }
        public DateTime Birthday { get; set; }
        public string Photo { get; set; }
        public string Gender { get; set; }
        public string Nation { get; set; }
        public string[] Interest { get; set; }
        public string Introduction { get; set; }
        public override int GetHashCode()
        {
            return base.GetHashCode();
        }
        public override bool Equals(object obj)
        {
            if (CardID == ((User)obj).CardID)
            {
                return true;
            }
            return base.Equals(obj);
        }
    }
    public class Order
    {
        public Order()
        {
            this.OrderDetailsList = new List<OrderDetails>();
        }
        public string OrderId { get; set; }
        public string CustomerId { get; set; }
        public DateTime OrderDate { get; set; }
        public IList<OrderDetails> OrderDetailsList { get; set; }

        public override int GetHashCode()
        {
            return base.GetHashCode();
        }
        public override bool Equals(object obj)
        {
            if (this.OrderId == ((Order)obj).OrderId)
            {
                return true;
            }
            return base.Equals(obj);
        }
        public string Link
        {
            get
            {
                return "http://www.google.com";
            }
        }
    }
    public class OrderDetails
    {
        public int Id { get; set; }
        public string OrderId { get; set; }
        public string ProductId { get; set; }
        public float UnitPrice { get; set; }
        public float Quantity { get; set; }
        public string StateType { get; set; }
        public override int GetHashCode()
        {
            return base.GetHashCode();
        }
        public override bool Equals(object obj)
        {
            if (this.Id == ((OrderDetails)obj).Id)
            {

            }
            return base.Equals(obj);
        }
    }
    public class Database
    {
        static IList<User> users = new List<User>();
        static IList<Order> orders = new List<Order>();
        static Database()
        {
            Order order = new Order();
            order.OrderId = "1";
            order.CustomerId = "DefaultCustomer";
            order.OrderDate = DateTime.Now;
            order.OrderDetailsList.Add(new OrderDetails()
            {
                Id = 1,
                OrderId = "1",
                ProductId = "Product1",
                Quantity = 2,
                UnitPrice = 2
            });
            orders.Add(order);
        }
        public static void AddUser(User user)
        {
            users.Add(user);
        }
        public static User[] GetUsers()
        {
            return users.ToArray();
        }
        public static User[] GetUsers(string name)
        {
            return users.Where(u => u.Name == name).ToArray();
        }
        public static User GetUser(string cardId)
        {
            return users.Where(u => u.CardID == cardId).FirstOrDefault();
        }
        public static void DeleteUser(string cardId)
        {
            User user = new User() { CardID = cardId };
            users.Remove(user);
        }

        public static void AddOrder(Order order)
        {
            orders.Add(order);
        }
        public static Order[] GetOrders()
        {
            return orders.ToArray();
        }
        public static Order GetOrder(string orderId)
        {
            return orders.Where(o => o.OrderId == orderId).First();
        }
        public static void DeleteOrder(string orderId)
        {
            orders.Remove(new Order { OrderId = orderId });
        }
    }
    public class DemoController : EverestControllerBase
    {
        public ActionResult Load()
        {
            JsonResult json = new JsonResult();
            json.Data = new ExtJsonReaderObject(Database.GetUsers(), Database.GetUsers().Count());
            return json;
        }
        public ActionResult Submit([Bind(Prefix = "")]User user, [Bind(Prefix = "oldData")]User oldUser, bool add, bool closeForm)
        {
            JsonResultData resultData = new JsonResultData() { closeForm = closeForm };
            if (add)
            {
                Database.AddUser(user);
            }
            else
            {
                //update  user 
                //old user will contains the old user data
                Database.DeleteUser(oldUser.CardID);
                Database.AddUser(user);
            }


            JsonResult result = Json(resultData);
            return result;
        }
        public ActionResult Delete(string OrderId)
        {            
            Database.DeleteOrder(OrderId);
            return null;
        }
        public ActionResult Details(string CardID)
        {
            JsonResult josn = new JsonResult();
            josn.Data = new JsonResultData() { data = Database.GetUser(CardID), success = true };
            return josn;
        }
        /// <summary>
        /// Gets the province.
        /// </summary>
        /// <returns></returns>
        public ActionResult GetProvince()
        {
            List<ComboboxItem> items = new List<ComboboxItem>();
            items.Add(new ComboboxItem("Fujian", "Fujian"));
            items.Add(new ComboboxItem("Guangdong", "Guangdong"));
            items.Add(new ComboboxItem("Beijing", "Beijing"));
            JsonResult json = new JsonResult();
            json.Data = items.ToArray();
            return json;
        }

        public ActionResult TreeNodeFromDemo()
        {
            IList<IDictionary<string, object>> treeNodes = new List<IDictionary<string, object>>();
            treeNodes.Add((new TreeNode() { Id = "TreeNodeFromDemo", Text = "TreeNodeFromDemoController", Leaf = true }).Attributes);
            JsonResult result = new JsonResult();
            result.Data = treeNodes.ToArray();
            return result;
        }

        #region order
        public ActionResult GetOrders()
        {
            JsonResult json = new JsonResult();
            json.Data = new ExtJsonReaderObject(Database.GetOrders(), Database.GetOrders().Count());
            return json;
        }
        public ActionResult GetOrder(string OrderId)
        {
            JsonResult json = new JsonResult();
            json.Data = new JsonResultData() { data = Database.GetOrder(OrderId), success = true };
            return json;
        }
        public ActionResult SubmitOrder([Bind(Prefix = "")] Order newOrder, [Bind(Prefix = "oldData")]Order oldOrder, bool closeForm, bool add)
        {
            var result = new JsonResultData() { data = null, success = true };
            try
            {
                Order order = null;
                if (add)
                {
                    order = new Order();
                    order.OrderId = Guid.NewGuid().ToString();
                    order.CustomerId = newOrder.CustomerId;
                    order.OrderDate = newOrder.OrderDate;
                    OrderDetails[] orderDetails = Request.Form["OrderDetailsList"].DeserializeJSON<OrderDetails[]>();
                    foreach (OrderDetails item in orderDetails)
                    {
                        item.Id = order.OrderDetailsList.Count() + 1;
                        order.OrderDetailsList.Add(item);
                    }
                    Database.AddOrder(order);
                }
                else
                {
                     order = Database.GetOrder(oldOrder.OrderId);
                    order.CustomerId = newOrder.CustomerId;
                    order.OrderDate = newOrder.OrderDate;
                    OrderDetails[] orderDetails = Request.Form["OrderDetailsList"].DeserializeJSON<OrderDetails[]>();
                    foreach (OrderDetails item in orderDetails)
                    {
                        if (item.StateType == "ADD")
                        {
                            item.Id = order.OrderDetailsList.Count() + 1;
                            order.OrderDetailsList.Add(item);
                        }
                        else if (item.StateType == "DELETE")
                        {
                            order.OrderDetailsList.Remove(new OrderDetails() { Id = item.Id });
                        }
                        else
                        {
                            OrderDetails details = order.OrderDetailsList.Where(d => d.Id == item.Id).First();
                            details.Quantity = item.Quantity;
                            details.UnitPrice = item.UnitPrice;
                            details.ProductId = item.ProductId;
                        }
                    }
                }
                if (!closeForm)
                {
                    return GetOrder(order.OrderId);
                }
            }
            catch (Exception e)
            {
                result.message = e.Message;
            }

            return Json(result);
        }
        #endregion

        public ActionResult ServerSideValidation()
        {
            JsonResultData result = new JsonResultData() { success = false };

            Exception ex = new Exception("Throw a exception.");
            //result.exception = ex;
            result.AddError("Name", "Invalide name value");
            result.AddError("Checked", "Invalide Checked value");
            throw ex;
            return Json(result);
        }
    }
}

