﻿/*
Kooboo is a content management system based on ASP.NET MVC framework. Copyright 2009 Yardi Technology Limited.

This program is free software: you can redistribute it and/or modify it under the terms of the
GNU General Public License version 3 as published by the Free Software Foundation.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program.
If not, see http://www.kooboo.com/gpl3/.
*/
/// <reference path="ext/3.0/ext-base.js" />
/// <reference path="ext/3.0/ext-all-debug.js" />
/// <reference path="Admin.js" />
/// <reference path="Resources.js" />

Everest.showErrorMessage = function(message, title) {
    Ext.MessageBox.show({
        title: title,
        msg: message,
        buttons: Ext.MessageBox.OK,
        icon: Ext.MessageBox.ERROR,
        width: 500
    });
}
Everest.showWarning = function(message, title) {
    Ext.MessageBox.show({
        title: title,
        msg: message,
        buttons: Ext.MessageBox.OK,
        icon: Ext.MessageBox.WARNING,
        width: 500
    });
}
Everest.showMessage = function(message, title) {
    Ext.MessageBox.show({
        title: title,
        msg: message,
        buttons: Ext.MessageBox.OK,
        icon: Ext.MessageBox.INFO
    });
}

Everest.GridPanel = function(config) {
    if (config.queryables) {
        this.queryField = new Array();
        Ext.each(config.queryables, function(q) {
            q = Ext.apply(q, { enableKeyEvents: true, width: 100, emptyText: q.fieldLabel, allowBlank: true, dataIndex: q.name, name: 'query_' + q.name, rightLabel: "" });
            this.queryField.push(Ext.ComponentMgr.create(q));
        }, this);
    }
    if (config.enableTopbar != false && config.tbar == undefined) {
        this.showDetailsButton = new Ext.Toolbar.Button({
            text: Resources.showDetails,
            handler: this.showDetails,
            iconCls: 'ShowDetail',
            tooltip: Resources.showDetails,
            scope: this
        });
        this.deleteButton = new Ext.Toolbar.Button({
            text: Resources.deleteText,
            handler: this.onDelete,
            iconCls: 'Delete',
            disabled: false,
            tooltip: Resources.deleteText,
            scope: this
        });
        config.tbarItems = Ext.combine([this.showDetailsButton, "-"], config.tbarItems || []);
        if (config.addable != false) {
            this.addButton = new Ext.Toolbar.Button({
                text: Resources.addNew,
                handler: this.onAdd,
                iconCls: 'Add',
                tooltip: Resources.addNew,
                scope: this
            });
            config.tbarItems = Ext.combine(['-', this.addButton, "-"], config.tbarItems || []);
        }
        config.tbarItems = Ext.combine(config.tbarItems, ['-', this.deleteButton]);

        if (this.queryField)
            config.tbarItems = Ext.combine(config.tbarItems, this.queryField);
        this.topBar = new Ext.PagingToolbar({
            pageSize: 50,
            store: config.store,
            displayInfo: true,
            beforePageText: Resources.beforePageText,
            afterPageText: Resources.afterPageText,
            displayMsg: Resources.pagingToolbar_Msg,
            emptyMsg: Resources.pagingToolbar_EmptyMsg,

            items: config.tbarItems
        });

        config = Ext.apply(config, {
            tbar: this.topBar
        });
        if (config.pageSize)
            this.topBar.pageSize = config.pageSize;
    }
    config = Ext.apply(config, {
        plugins: this.getContextMenu(),
        loadMask: true,
        listeners: { rowdblclick: this.showDetails, scope: this }
    });
    Everest.GridPanel.superclass.constructor.call(this, config);
    this.getStore().on('beforeload', this.onBeforeload, this);
    this.on("render", this.onMyRender, this);
}
Ext.extend(Everest.GridPanel, Ext.grid.GridPanel, {
    canCopy: true,
    onBeforeload: function(store, options) {
        if (this.queryField) {
            Ext.each(this.queryField, function(q) {
                this.store.baseParams[q.dataIndex] = q.getValue();
            }, this)
        }
    },
    onMyRender: function() {
        if (this.queryField) {
            Ext.each(this.queryField, function(q) {
                if (q.xtype == 'combo') {
                    q.addListener('collapse', this.refresh, this);
                    return;
                }
                else {
                    q.on('keydown', function(f, e) {
                        if (e.getCharCode() == 13) {
                            this.refresh();
                        }
                    }, this);
                }

            }, this);
        }
    },
    getContextMenu: function() {
        return new Everest.plugins.GridContextMenu();
    },
    onAdd: function() {
        this.openNewForm();
    },
    openNewForm: function(formLoadParams, copyNew) {
        var params = { popup: true, formTitle: this.title, copyNew: copyNew, params: { schema: this.schemaName, formType: 'form' }, opener: this, formLoadParams: formLoadParams };
        Global.Instance.getForm(params);
    },
    load: function(option) {
        //如果传入option，则要指定option.params.start 等分页信息，如果没有指定分页信息会显示NaN    
        this.getStore().load(option);
    },
    copyNew: function() {
        if (this.canCopy) {
            var data = this.getSelectedData();
            if (data != null)
                this.openNewForm(data, true);
        }
    },
    showDetails: function() {
        var data = this.getSelectedData();
        if (data != null)
            this.openNewForm(data, false);
    },
    getSelectedData: function() {
        var selectedRow = this.getSelectionModel();
        if (selectedRow.getCount() == 0) {
            return null;
        }
        return selectedRow.getSelected().data;
    },
    onDelete: function() {
        var selectionModel = this.getSelectionModel();
        var selections = selectionModel.getSelections();
        if (selections.length > 0) {
            var p = "action=delete";
            Ext.each(selections, function(r) {
                p = p + "&" + Ext.urlEncode(r.data);
            }, this);
            Ext.MessageBox.confirm(Resources.confirmTitle, Resources.confirmDeleteMsg, function(btn) {
                if (btn == 'yes') {
                    Ext.Ajax.request({
                        url: this.deleteUrl,
                        success: this.onDeleteComplete,
                        failure: this.onDeleteComplete,
                        params: p,
                        scope: this
                    })
                }
            }, this);
        }
    },
    onDeleteComplete: function(response) {
        this.refresh();
    },
    refresh: function() {
        if (this.topBar && this.topBar.onClick)
            this.topBar.onClick(this.topBar.refresh);
        else {
            this.load();
        }
    },
    setReadonly: function(readonly) {
        if (this.deleteButton)
            this.deleteButton.setDisabled(readonly);
        this.readonly = readonly;
    },
    getReadonly: function() {
        return this.readonly;
    },
    setPageSize: function(pageSize) {
        this.tbar.pageSize = pageSize;
    },
    destroy: function() {
        if (this.opener != null) {
            var opener = this.opener;
            delete opener.openedPanel;
            delete this.opener;
        }
        Everest.GridPanel.superclass.destroy.call(this);
    }
});

Everest.FormPanel = function(config) {
    this.params = Ext.apply(config.params || {}, { add: true });
    if (config.enableTopbar != false && config.tbar == undefined) {
        this.saveAndCloseButton = new Ext.Toolbar.Button({
            text: Resources.form_saveAndClose,
            iconCls: 'Save_Close',
            tooltip: Resources.form_saveAndClose,
            handler: function(btn) {
                this.save(true);
            },
            scope: this
        });
        this.saveButton = new Ext.Toolbar.Button({
            text: Resources.form_save,
            iconCls: 'Save',
            tooltip: Resources.form_save,
            handler: function(btn) {
                this.save(false);
            },
            scope: this
        });
        this.resetButton = new Ext.Toolbar.Button({
            text: Resources.form_reset,
            iconCls: 'Reset',
            tooltip: Resources.form_reset,
            handler: function(btn) {
                this.reset();
            },
            scope: this
        });
        this.closeButton = new Ext.Toolbar.Button({
            text: Resources.form_close,
            iconCls: 'Close',
            tooltip: Resources.form_close,
            handler: function(btn) {
                this.close();
            },
            scope: this
        });
        config.tbarItems = Ext.combine([this.saveAndCloseButton, "-",
            this.saveButton], config.tbarItems || []);
        config.tbarItems = Ext.combine(config.tbarItems, ['-', this.resetButton, '-', this.closeButton]);
        this.tbar = new Ext.Toolbar({
            items: config.tbarItems
        });
        config = Ext.apply(config, {
            tbar: this.tbar
        });
    }
    //    var d = new Date();
    //    //extjs 3.0 does not support the statusbar
    //    this.statusBar = new Ext.ux.StatusBar({
    //        defaultText: Resources.statusText_Add,
    //        layout: 'fit',
    //        text: Resources.statusText_Add,
    //        // any standard Toolbar items:
    //        items: ['-', d.dateFormat("H:i:s")]

    //    });
    config = Ext.apply(config, {
        autoScroll: true,
        trackResetOnLoad: true,
        monitorValid: false,
        //        bbar: this.statusBar,
        keys: [{
            key: 's',
            ctrl: true,
            handler: function() { this.save(false); },
            stopEvent: true,
            scope: this
        }
            ]
    });
    Everest.FormPanel.superclass.constructor.call(this, config);
    this.getForm().on('actioncomplete', this.onActionCompleted, this);
    this.getForm().on("beforeaction", this.onBeforeaction, this);
    this.getForm().on('actionfailed', this.onActionFailed, this);
    this.getForm().on('beforeSetValues', this.onBeforeSetValues, this);
    this.on("render", this.onMyRender, this);
}
Ext.extend(Everest.FormPanel, Ext.form.FormPanel, {
    onBeforeSetValues: function(basicForm, data) {
    },
    onBeforeaction: function(basicForm, action) {
        this.mask();
    },
    onActionFailed: function(form, action) {
        this.hideMask();
        //        if (action.result.exception)
        //            Everest.showErrorMessage(Ext.encode(action.result.exception), Everest.language.catchException);
    },
    onActionCompleted: function(form, action) {
        this.hideMask();
        if (action.type == 'submit') {
            try {
                if (this.opener && this.opener.refresh)
                    this.opener.refresh();
                if (this.opener && this.opener.reload)
                    this.opener.reload();
            }
            catch (e) {
            }
            this.setChanged(false);
            if (action.result.closeForm == true)
                this.close();
            if (action.result.data) {
                this.getForm().setValues(action.result.data);
            }
        }
        if (action.result.data) {
            if (!this.copyNew)
                this.setUpdateState(action.result.data);
            if (action.result.data.FormTitle) {
                this.setTitle(action.result.data.FormTitle);
            }
        }
        this.setStatusText();
    },
    getChanged: function() {
        return this.changed;
    },
    setChanged: function(value) {
        this.changed = value;
    },
    submit: function(options) {
        if (Ext.ux.TinyMCE)
            tinyMCE.triggerSave();
        if (Ext.ux.radEditor)
            Ext.ux.radEditor.triggerSave();
        if (Ext.ux.CodePress)
            Ext.ux.CodePress.triggerSave();
        if (this.getForm().isValid())
            this.getForm().submit(Ext.apply(options || {}, { params: this.params }));
    },
    save: function(closeForm) {
        this.params.closeForm = closeForm;
        this.submit();
    },
    mask: function() {
        if (this.bwrap) {
            if (!this.loadMask)
                this.loadMask = new Ext.LoadMask(this.bwrap, { msg: Resources.mask_msg, removeMask: true });
            this.loadMask.show();
            //request timeout. auto hide loadMask.
            this.hideMask.defer(30000, this);
        }
    },
    hideMask: function() {
        if (this.loadMask) {
            this.loadMask.hide();
        }
    },
    load: function(oldData, copyNew) {
        if (this.loadUrl)
            this.getForm().load({ url: this.loadUrl, params: oldData });
        this.copyNew = copyNew;
    },
    setUpdateState: function(oldData) {
        this.setOldData(oldData);
        this.params = this.params || {};
        for (var p in oldData)
            this.params["oldData." + p] = oldData[p];
        this.items.each(function(i) {
            if (i.modifiable == false)
                i.setDisabled(true);
        }, this);
    },
    onFieldChange: function(field, newValue, oldValue) {
        this.setChanged(true);
    },
    setWindow: function(window) {
        this.window = window;
        this.window.on("beforeclose", this.onBeforeClose, this);
    },
    onBeforeClose: function(formPanel) {
        if (this.getChanged() == true) {
            Ext.MessageBox.confirm(Resources.confirmTitle, Resources.confirmClose, function(btn) {
                if (btn == 'yes') {
                    this.window.un("beforeclose", this.onBeforeClose, this);
                    this.close(true);
                }

            }, this);
            return false;
        }
        else
            return true;
    },
    close: function(refresh) {
        if (refresh == true) {
            this.refreshParent();
        }
        this.closeForm();
    },
    closeForm: function() {
        if (this.window) {
            this.window.close();
        }
        else
            Global.Instance.closeForm(this);
    },
    refreshParent: function() {
        if (this.opener && this.opener.refresh)
            this.opener.refresh();
    },
    reset: function() {
        if (this.isNew()) {
            this.getForm().reset();
        }
        else {
            this.load(this.oldData);
        }
    },
    onMyRender: function() {
        this.getForm().items.each(function(f) {
            f.nodeId = this.nodeId;
            f.application = this.application;
            if (f instanceof Ext.form.FileUploadField) {
                f.on("fileselected", this.onFieldChange, this);
            }
            else {
                // The htmleditor don't have change event.
                if ((f.getXType() == "htmleditor") || (f.getXType() == 'tinymce')) {
                    this.setChanged(true);
                }
                else
                    f.on("change", this.onFieldChange, this);
            }
            if (f.depency) {
                if (f.depency instanceof Array) {
                    for (var i = 0; i < f.depency.length; i++) {
                        var depencyObj = f.depency[i];
                        var field = this.getForm().findField(depencyObj.fieldId);
                        if (field)
                            f.setDepencyControl({ paramName: depencyObj.param, control: field });
                    }
                }
                else {
                    var field = this.getForm().findField(f.depency.fieldId);
                    if (field)
                        f.setDepencyControl({ paramName: f.depency.param, control: field });
                }
            }
        }, this);
    },
    setReadonly: function(readonly) {
        if (this.saveAndCloseButton)
            this.saveAndCloseButton.setDisabled(readonly);
        if (this.saveButton)
            this.saveButton.setDisabled(readonly);
        this.readonly = readonly;
        this.setStatusText();
    },
    setStatusText: function() {
        if (this.readonly == true) { }
        //            this.statusBar.setText(Resources.statusText_Readonly);
        else {
            if (this.isNew() == true) {
                //                this.statusBar.setText(Resources.statusText_Add);
            }
            else {
                //                this.statusBar.setText(Resources.statusText_Update);
            }
        }
    },
    getReadonly: function() {
        return this.readonly;
    },
    isNew: function() {
        return this.params["add"];
    },
    setOldData: function(oldData) {
        this.oldData = oldData;
        if (this.oldData) {
            this.params["add"] = false;
        }
        else
            this.params["add"] = true;
    },
    destroy: function() {
        if (this.opener != null) {
            var opener = this.opener;
            opener.openedPanel = null;
            delete opener.openedPanel;
            delete this.opener;
        }
        Everest.FormPanel.superclass.destroy.call(this);
    }
})

Ext.override(Ext.form.BasicForm, {
    findField: function(id) {
        var field = this.items.get(id);
        if (!field) {
            this.items.each(function(f) {
                if ((f.dataIndex == id || f.id == id || f.getName() == id)) {
                    field = f;
                    return false;
                }
            });
        }
        return field || null;
    }
})
//提交数据时会传带删除标志的行
Everest.EditorGridPanel = function(config) {
    config = Ext.apply(config, {
        sm: new Ext.grid.RowSelectionModel(),
        viewConfig: {
            getEditorParent: function() {
                return Ext.getBody();
            }
        }
    });
    if (config.enableTopbar != false) {
        var toolBarItems = [{
            text: Resources.grid_submit,
            hidden: !config.submitUrl,
            handler: function(btn) {
                this.submit();
            },
            scope: this
        }, '-', {
            text: Resources.grid_insertItem,
            handler: function(btn) {
                this.onInsertItem();
            },
            scope: this
        }, '-', {
            text: Resources.grid_deleteItem,
            handler: function(btn) {
                this.onDeleteItem();
            },
            scope: this
        }
            ];
        toolBarItems = Ext.combine(toolBarItems, config.tbarItems || {});
        if (config.store.url) {
            this.topBar = new Ext.PagingToolbar({
                items: toolBarItems,
                store: config.store
            });
        }
        else {
            this.topBar = new Ext.Toolbar({
                items: toolBarItems
            });
        }
        config = Ext.apply(config, {
            tbar: this.topBar
        });
    }
    Everest.EditorGridPanel.superclass.constructor.call(this, config);
    this.getStore().on("remove", this.onRemoved, this);
    this.on("render", this.onMyRender, this);
    this.on("beforeedit", this.onBeforeEdit);
}
Ext.extend(Everest.EditorGridPanel, Ext.grid.EditorGridPanel, {
    getValue: function() {
        var data = this.getModifiedRecords();
        var v = {}
        v[this.dataIndex || this.getDefaultParamName()] = Ext.util.JSON.encode(data);
        return v;
    },
    getDefaultParamName: function() {
        return "items";
    },
    submit: function() {
        Ext.Ajax.request({
            url: this.submitUrl,
            success: function(o) {
                this.load();
            },
            failure: function() {
                Everest.showErrorMessage("request failure!");
            },
            params: this.getValue(),
            scope: this
        });
    },
    onDeleteItem: function() {
        Ext.MessageBox.confirm(Resources.confirmTitle, Resources.confirmDeleteMsg, function(btn) {
            if (btn == 'yes') {
                var sm = this.getSelectionModel();
                Ext.each(sm.getSelections(), function(r) {
                    //can not be removed.
                    if (r.fixed != true) {
                        this.getStore().remove(r);
                    }
                }, this);
            }
        }, this);
    },
    onInsertItem: function(data) {
        var store = this.getStore();
        //        var record = this.getSelectionModel().getSelected();
        //        var index = store.getCount();
        //        if (record) {
        //            index = store.indexOf(record);
        //        }
        var orec, row;
        if (store.recordType) {
            orec = new store.recordType();
            if (!data) {
                orec.data = {};
                orec.fields.each(function(field) {
                    orec.data[field.name] = field.defaultValue;
                });
                orec.data.StateType = 'ADD';
                orec.commit();
            }
            else {
                orec.data = data;
                orec.data.StateType = 'ADD';
            }
            store.add(orec);

            row = store.indexOf(orec);
            if (row && undefined !== this.editOnAddCol) {
                this.startEditing(row, this.editOnAddCol);
            }
        }
    },
    onRemoved: function(store, record, index) {
        if (record.data.StateType == 'ADD') {
            record.data.StateType = 'Rejected';
            return;
        }
        if (this.removedRecords == null)
            this.removedRecords = new Array();
        record.data.StateType = 'DELETE';
        this.removedRecords.push(record);
    },
    getModifiedRecords: function() {
        var modified = this.getStore().getModifiedRecords();
        if (this.removedRecords) {
            Ext.each(this.removedRecords, function(r) {
                modified.push(r);
            });
        }
        delete this.removedRecords;
        var data = new Array();
        Ext.each(modified, function(r) {
            //如果移动了，那么状态会被Rejected...       
            if (r.data.StateType != 'Rejected') {
                if (!r.data.StateType)
                    r.data.StateType = "UPDATE";
                data.push(r.data);
            }
        }, this);
        return data;
    },
    onMyRender: function(com) {
        //enable c.editor field can depency by other control.
        this.getColumnModel().getColumnsBy(function(c) {
            if (c.editor) {
                c.editor.nodeId = this.nodeId;
                c.editor.application = this.application;
                if (c.editor.depency) {
                    this.getColumnModel().getColumnsBy(function(p) {
                        if (c.editor.depency instanceof Array) {
                            for (var i = 0; i < c.editor.depency.length; i++) {
                                var depencyObj = c.editor.depency[i];
                                if (depencyObj.fieldId == p.dataIndex) {
                                    f.setDepencyControl({ paramName: depencyObj.param, control: p.editor });
                                }
                            }
                        }
                        else {
                            if (c.editor.depency.fieldId == p.dataIndex) {
                                f.setDepencyControl({ paramName: f.depency.param, control: p.editor });
                            }
                        }
                    }, this);
                }
            }
        }, this);
        this.load();
    },
    beforeload: function(store, options) {
        if (this.queryables) {
            Ext.each(this.queryField, function(q) {
                this.store.baseParams[q.fieldName] = q.getValue();
            }, this)
        }
    },
    load: function() {
        if (this.getStore().url)
            this.getStore().load();
    },
    refresh: function() {
        if (this.topBar.onClick)
            this.topBar.onClick("refresh");
    },
    onBeforeEdit: function(e) {
        if (e.record.fixed == true) {
            e.cancel = true;
        }
    }
})
Everest.MasterFormPanel = function(config) {
    Everest.MasterFormPanel.superclass.constructor.call(this, config);

    if (config.childForms && config.childForms.length > 0) {
        this.formsContainer = new Ext.TabPanel({ autoWidth: true, height: 250, activeTab: 0 });
        Ext.each(config.childForms, function(f) {
            this.formsContainer.add(f);
            f.parentForm = this;
            //set form changed by the editor grid panel.
            if (f instanceof Ext.grid.EditorGridPanel) {
                f.getStore().on("update", function() { this.setChanged(true); }, this);
                f.getStore().on("add", function(store, records, index) {
                    //determine if there has fixed records
                    //if there has fixed records,will be regarded as in initializing the grid.
                    var isFixed = false;
                    for (var i = 0; i < records.length; i++) {
                        isFixed = records[i].fixed;
                        if (isFixed) break;
                    }
                    this.setChanged(!isFixed);
                }, this);
                f.getStore().on("remove", function() { this.setChanged(true); }, this);
            }
        }, this);
        this.add(this.formsContainer);
        this.on("activate", function() { if (this.formsContainer) this.formsContainer.doLayout(); }, this);
    }
    this.on('resize', function() { if (this.formsContainer) this.formsContainer.doLayout(); });
    this.getForm().on("beforeaction", this.onBeforeAction, this);
}
Ext.extend(Everest.MasterFormPanel, Everest.FormPanel, {
    onActionCompleted: function(basicForm, action) {
        //            if (action.type == "load") {
        //                if (this.getDataForms()) {
        //                    this.getDataForms().each(function(di) {
        //                        var data = {};
        //                        data.Rows = action.result.data[di.dataIndex];
        //                        if (data.Rows) {
        //                            data.TotalCount = data.Rows.length;
        //                            di.store.loadData(data);
        //                        }
        //                    });
        //                }
        //            }
        Everest.MasterFormPanel.superclass.onActionCompleted.call(this, basicForm, action);
    },
    onBeforeSetValues: function(basicForm, data) {
        if (this.getDataForms()) {
            this.getDataForms().each(function(di) {
                var gridData = {};
                gridData.Rows = data[di.dataIndex];
                if (gridData.Rows) {
                    gridData.TotalCount = gridData.Rows.length;
                    di.store.loadData(gridData);
                }
            });
        }
        Everest.MasterFormPanel.superclass.onBeforeSetValues.call(this, basicForm, data);
    },
    getDataForms: function() {
        if (this.formsContainer)
            return this.formsContainer.items;
    },
    getDataFormByDataIndex: function(dataIndex) {
        if (this.getDataForms()) {
            this.getDataForms().each(function(i) {
                if (i.dataIndex == dataIndex)
                    return i;
            }, this);
        }
        return null;
    },
    onBeforeAction: function(basicForm, action) {
        if (action.type == 'submit') {
            basicForm.baseParams = basicForm.baseParams || {};
            if (this.getDataForms()) {
                this.getDataForms().each(function(i) {
                    if (i.getValue) {
                        basicForm.baseParams = Ext.apply(basicForm.baseParams, i.getValue());
                    }
                }, this);
            }
        }
    },
    setReadonly: function(readonly) {
        Everest.MasterFormPanel.superclass.setReadonly.call(this, readonly);
        //set the child form toolbar disable
        if (this.formsContainer) {
            this.formsContainer.items.each(function(f) {
                var tbar = f.getTopToolbar();
                if (tbar) {
                    tbar.setDisabled(readonly);
                }
            });
        }
    },
    doLayout: function() {
        Everest.MasterFormPanel.superclass.doLayout.call(this, arguments);
    }
})
//提交数据时，不会传带删除标志的行。
Everest.DetailsEditorGrid = function(config) {
    config.tbarItems = [{
        text: '',
        icon: Resources.upIcon,
        tooltip: Resources.up,
        handler: this.Up,
        scope: this
    }, {
        text: '',
        icon: Resources.downIcon,
        tooltip: Resources.down,
        handler: this.Down,
        scope: this
    }
];
    Everest.DetailsEditorGrid.superclass.constructor.call(this, config);
}
Ext.extend(Everest.DetailsEditorGrid, Everest.EditorGridPanel, {
    getModifiedRecords: function() {
        var modified = new Array();
        this.getStore().each(function(r) {
            if (r.fixed != true) {
                modified.push(r);
            }
        });
        var data = new Array();
        Ext.each(modified, function(r) {
            delete r.data.StateType;
            data.push(r.data);
        }, this);
        return data;
    },
    Up: function() {
        this.moveSelectedRow(this, -1);
    },
    moveSelectedRow: function(grid, direction) {
        var record = grid.getSelectionModel().getSelected();
        if (!record) {
            return;
        }
        var index = grid.getStore().indexOf(record);
        if (direction < 0) {
            index--;
            if (index < 0) {
                return;
            }
        } else {
            index++;
            if (index >= grid.getStore().getCount()) {
                return;
            }
        }
        grid.getStore().remove(record);
        grid.getStore().insert(index, record);
        grid.getSelectionModel().selectRow(index, true);
    },
    Down: function() {
        this.moveSelectedRow(this, 0);
    }
})
Everest.DraggableDetailsEditorGrid = function(config) {
    config = Ext.apply(config, {
        ddGroup: 'mygridDD',
        sm: new Ext.grid.RowSelectionModel({ singleSelect: true }),
        enableDragDrop: true, // enable drag and drop of grid rows
        listeners: {
            "render": {
                scope: this,
                fn: function(grid) {

                    // Enable sorting Rows via Drag & Drop
                    // this drop target listens for a row drop
                    //  and handles rearranging the rows

                    var ddrow = new Ext.dd.DropTarget(grid.container, {
                        ddGroup: 'mygridDD',
                        copy: false,
                        notifyDrop: function(dd, e, data) {

                            var ds = grid.store;

                            // NOTE:
                            // you may need to make an ajax call here
                            // to send the new order
                            // and then reload the store


                            // alternatively, you can handle the changes
                            // in the order of the row as demonstrated below

                            // ***************************************

                            var sm = grid.getSelectionModel();
                            var rows = sm.getSelections();
                            if (dd.getDragData(e)) {
                                var cindex = dd.getDragData(e).rowIndex;
                                if (typeof (cindex) != "undefined") {
                                    for (i = 0; i < rows.length; i++) {
                                        ds.remove(ds.getById(rows[i].id));
                                    }
                                    ds.insert(cindex, data.selections);
                                    sm.clearSelections();
                                }
                            }

                            // ************************************
                        }
                    })

                    // load the grid store
                    //  after the grid has been rendered
                    //store.load();
                }
            }
        }
    });

    Everest.DraggableDetailsEditorGrid.superclass.constructor.call(this, config);
};
Ext.extend(Everest.DraggableDetailsEditorGrid, Everest.DetailsEditorGrid, {
});