﻿/*
Kooboo is a content management system based on ASP.NET MVC framework. Copyright 2009 Yardi Technology Limited.

This program is free software: you can redistribute it and/or modify it under the terms of the
GNU General Public License version 3 as published by the Free Software Foundation.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program.
If not, see http://www.kooboo.com/gpl3/.
*/
/// <reference path="ext/3.0/ext-base.js" />
/// <reference path="ext/3.0/ext-all-debug.js" />


Ext.namespace('Everest');
Ext.namespace('Everest.plugins');
Everest.plugins.language = {
    tabClose: 'Close',
    tabCloseOthers: 'Close Others',
    tabCloseAll: 'Close All',
    tabRefresh: 'Refresh',
    gridCopyCell: 'Copy Cell',
    gridCopySelectedRow: 'Copy Selected Row',
    gridRefresh: 'Refresh',
    gridSelectAll: 'Select All',
    gridSelectCurrent: 'Select Row',
    gridClearAll: 'Clear All Selected',
    gridReverseSelect: 'Reverse Select'
}
Everest.plugins.TabCloseMenu = function() {
    var tabs, menu, ctxItem;
    this.init = function(tp) {
        tabs = tp;
        tabs.on('contextmenu', onContextMenu);
    }

    function onContextMenu(ts, item, e) {
        if (!menu) { // create context menu on first right click            
            menu = new Ext.menu.Menu([{
                id: tabs.id + '-close',
                text: Everest.plugins.language.tabClose,
                iconCls: 'icon-close',
                handler: function() {
                    tabs.remove(ctxItem);
                }
            }, {
                id: tabs.id + '-close-others',
                text: Everest.plugins.language.tabCloseOthers,
                iconCls: 'icon-close',
                handler: function() {
                    tabs.items.each(function(item) {
                        if (item.closable && item != ctxItem) {
                            tabs.remove(item);
                        }
                    });
                }
            }, {
                id: tabs.id + '-close-all',
                text: Everest.plugins.language.tabCloseAll,
                iconCls: 'icon-close',
                handler: function() {
                    tabs.items.each(function(item) {
                        if (item.closable) {
                            tabs.remove(item);
                        }
                    });
                }
            }, {
                id: tabs.id + '-refresh',
                text: Everest.plugins.language.tabRefresh,
                iconCls: 'icon-refresh',
                handler: function() {
                    if (item.refresh) {
                        item.refresh();
                    }
                }
            }
                ]
);
        }
        ctxItem = item;
        var items = menu.items;
        items.get(tabs.id + '-close').setDisabled(!item.closable);
        var disableOthers = true;
        tabs.items.each(function() {
            if (this != item && this.closable) {
                disableOthers = false;
                return false;
            }
        });
        items.get(tabs.id + '-close-others').setDisabled(disableOthers);
        menu.showAt(e.getXY());
    }
}

Everest.plugins.GridContextMenu = function(config) {
    this.config = config || {};
    Ext.apply(this, this.config);
}
Everest.plugins.GridContextMenu.prototype = {
    init: function(grid) {
        this.grid = grid;
        this.grid.on('cellcontextmenu', this.onCellContextMenu, this);
        //        this.grid.on('rowcontextmenu', this.onRowContextMenu, this);
        this.grid.on('contextmenu', this.onContextMenu, this);
    },
    onCellContextMenu: function(grid, rowIndex, cellIndex, e) {
        this.currentCellIndex = cellIndex;
        this.currentRowIndex = rowIndex;
        this.grid.getSelectionModel().selectRow(this.currentRowIndex);
        if (!this.cellMenu) {
            this.cellMenu = new Ext.menu.Menu();
            this.addCellContextMenu(this.cellMenu);
        }
        this.cellMenu.showAt(e.getXY());
        e.stopEvent();
    },
    addCellContextMenu: function(menu) {
        menu.add({ text: Everest.plugins.language.gridRefresh, handler: this.onRefresh, scope: this });
        menu.add('-');
        var copycell = menu.add({ text: Everest.plugins.language.gridCopyCell, handler: this.onCopyCell, scope: this });
        var copyrow = menu.add({ text: Everest.plugins.language.gridCopySelectedRow, handler: this.onCopyRow, scope: this });
        menu.add('-');
        if (this.config.items != undefined) {
            for (var i = 0; i < this.config.items.length; i++) {
                this.addItem(menu, this.config.items[i]);
            }
        }
        menu.add('-');
        menu.add({ text: Everest.plugins.language.gridSelectAll, handler: this.selectAll, scope: this })
        //            item = menu.add({ id: this.grid.id + '-selectCurrent', text: Everest.plugins.language.gridSelectCurrent, handler: this.selectCurrent, scope: this });
        menu.add({ text: Everest.plugins.language.gridClearAll, handler: this.clearSelections, scope: this });
        //            item = menu.add({ id: this.grid.id + '-reverseSelect', text: Everest.plugins.language.gridReverseSelect, handler: this.reverseSelect, scope: this });
    },
    onContextMenu: function(e) {
        e.stopEvent();
    },
    onRowContextMenu: function(grid, rowIndex, e) {
        if (!this.rowMenu) {
            this.rowMenu = new Ext.menu.Menu({ plugins: new Everest.plugins.NonMenu() });
            this.addRowContextMenu(this.rowMenu);
        }
        this.currentRowIndex = rowIndex;
        this.grid.getSelectionModel().selectRow(this.currentRowIndex);
        this.rowMenu.showAt(e.getXY());
        e.stopEvent();
    },
    addRowContextMenu: function(menu) {
        menu.add({ text: Everest.plugins.language.gridRefresh, handler: this.onRefresh, scope: this });
        menu.add('-');
        if (this.config.items != undefined) {
            for (var i = 0; i < this.config.items.length; i++) {
                this.addItem(this.rowMenu, this.config.items[i]);
            }
        }
        menu.add('-');
        menu.add({ text: Everest.plugins.language.gridSelectAll, handler: this.selectAll, scope: this })
        //            item = menu.add({ id: this.grid.id + '-selectCurrent', text: Everest.plugins.language.gridSelectCurrent, handler: this.selectCurrent, scope: this });
        menu.add({ text: Everest.plugins.language.gridClearAll, handler: this.clearSelections, scope: this });
        //            item = menu.add({ id: this.grid.id + '-reverseSelect', text: Everest.plugins.language.gridReverseSelect, handler: this.reverseSelect, scope: this });
    },
    onCopyCell: function() {
        var cm = this.grid.getColumnModel();
        var recorde = this.grid.getStore().getAt(this.currentRowIndex);
        String.copyToClipboard(recorde.get(cm.getDataIndex(this.currentCellIndex)).toString());
    },
    onCopyRow: function() {
        var cm = this.grid.getColumnModel();
        var selectedRecordes = this.grid.getSelectionModel().getSelections();
        var s = "";
        Ext.each(selectedRecordes, function(r) {
            for (var i = 0; i < cm.getColumnCount(); i++) {
                var columnName = cm.getDataIndex(i);
                if (columnName != "") {
                    if (cm.getRenderer(i) != undefined) {
                        s += cm.getRenderer(i)(r.get(columnName)) + '\t';
                    }
                    else {
                        s += r.get(columnName).toString() + '\t';
                    }
                }
            }
            s += "\n";
        });
        String.copyToClipboard(s);
    },
    onRefresh: function() {
        if (this.grid.load)
            this.grid.load();
    },
    deleteCurrent: function(menuItem, e) {
        var row = this.grid.store.getAt(this.currentRowIndex);
        Ext.callback(this.config.deleteCurrentHandler, this.config.scope || this, [row]);
    },
    deleteSelections: function(menuItem, e) {
        var rows = this.grid.getSelectionModel().getSelections();
        Ext.callback(this.config.deleteSelectionsHandler, this.config.scope || this, [rows]);
    },
    clearSelections: function(menuItem, e) {
        this.grid.getSelectionModel().clearSelections();
    },
    selectAll: function(menuItem, e) {
        this.grid.getSelectionModel().selectAll();
    },
    selectCurrent: function(menuItem, e) {
        this.grid.getSelectionModel().selectRow(this.currentRowIndex);
    },
    clearSelections: function(menuItem, e) {
        this.grid.getSelectionModel().clearSelections();
    },
    deselectRow: function(menuItem, e) {
        this.grid.getSelectionModel().deselectRow(this.currentRowIndex);
    },
    reverseSelect: function(menuItem, e) {
        var count = this.grid.store.getCount();
        var selModel = this.grid.getSelectionModel();
        var s = new Array();
        for (var i = 0; i < count; i++) {
            if (!selModel.isSelected(i))
                s.push(i);
        }
        selModel.selectRows(s);
    },
    addItem: function(menu, item) {
        item.scope = this;
        menu.add(item);
    }
}
Everest.plugins.NonMenu = function() {
};
Everest.plugins.NonMenu.prototype = {
    init: function(component) {
        component.on('render', this.onRender);
    },
    onRender: function() {
        Ext.EventManager.on(this.getEl(), 'contextmenu', function(e) {
            e.stopEvent();
        });
    }
}
//扩展字符串类，增加拷贝到内存的方法
String.copyToClipboard = function(txt) {
    if (window.clipboardData) {
        window.clipboardData.clearData();
        window.clipboardData.setData("Text", txt);
    } else if (navigator.userAgent.indexOf("Opera") != -1) {
        window.location = txt;
    } else if (window.netscape) {
        try {
            netscape.security.PrivilegeManager.enablePrivilege("UniversalXPConnect");
        } catch (e) {
            alert("open 'about:config',set the 'signed.applets.codebase_principal_support' value to 'true'.");
        }
        var clip = Components.classes['@mozilla.org/widget/clipboard;1'].createInstance(Components.interfaces.nsIClipboard);
        if (!clip)
            return;
        var trans = Components.classes['@mozilla.org/widget/transferable;1'].createInstance(Components.interfaces.nsITransferable);
        if (!trans)
            return;
        trans.addDataFlavor('text/unicode');
        var str = new Object();
        var len = new Object();
        var str = Components.classes["@mozilla.org/supports-string;1"].createInstance(Components.interfaces.nsISupportsString);
        var copytext = txt;
        str.data = copytext;
        trans.setTransferData("text/unicode", str, copytext.length * 2);
        var clipid = Components.interfaces.nsIClipboard;
        if (!clip)
            return false;
        clip.setData(trans, null, clipid.kGlobalClipboard);
    }
}