﻿
/*
*
* Ext.ux.CodeMirror
* 
*/

Ext.namespace('Ext.ux');

//#region
Ext.ux.MyKeyNav = function(el, config) {
    this.el = el;
    //this.el = Ext.get(el);
    Ext.apply(this, config);
    if (!this.disabled) {
        this.disabled = true;
        this.enable();
    }
};

Ext.extend(Ext.ux.MyKeyNav, Ext.KeyNav, {
    enable: function() {
        if (this.disabled) {
            var self = this;
            if (this.isKeydown()) {
                this.scope.AttachEvent(this.el, 'keydown', (this.fn1 = function(e) {
                    self.relay(new Ext.EventObjectImpl(e));
                }));
            } else {
                this.scope.AttachEvent(this.el, 'keydown', (this.fn2 = function(e) {
                    self.prepareEvent(new Ext.EventObjectImpl(e));
                }));
                this.scope.AttachEvent(this.el, 'keypress', (this.fn3 = function(e) {
                    self.relay(new Ext.EventObjectImpl(e));
                }));
            }
            this.disabled = false;
        }
    },
    disable: function() {
        if (!this.disabled) {
            if (this.isKeydown()) {
                this.scope.DetachEvent(this.el, 'keydown', this.fn1);
            } else {
                this.scope.DetachEvent(this.el, 'keydown', this.fn2);
                this.scope.DetachEvent(this.el, 'keypress', this.fn3);
            }
            this.disabled = true;
        }
    }
});
//#endregion

//#region
Ext.ux.Selection = function(win) {
    this._window = win;
    this._document = win.document;
    this._isIE = Ext.isIE;
};

Ext.ux.Selection.prototype = {
    getRange: function() {
        if (!this._window) {
            return null;
        }
        if (this._document.selection && !window.opera) {
            return this._document.selection.createRange();
        } else {
            if (this._window.getSelection) {
                var selection = this._window.getSelection();
                if (!selection || selection.rangeCount < 1) {
                    return null;
                }
                var range = null;
                if (selection.getRangeAt) {
                    range = selection.getRangeAt(0);
                } else {
                    range = this._document.createRange();
                    range.setStart(selection.anchorNode, selection.anchorOffset);
                    range.setEnd(selection.focusNode, selection.focusOffset);
                }
                return range;
            }
        }
    },
    selectRange: function(range) {
        if (!range) {
            return;
        }
        var win = this._window;
        if (range.select) {
            range.select();
        } else {
            if (win.getSelection) {
                var selection = win.getSelection();
                if (selection.removeAllRanges) {
                    selection.removeAllRanges();
                    selection.addRange(range);
                } else {
                    var startNode = range.baseNode;
                    if (startNode == null) {
                        startNode = win.document.body;
                    }
                    var endNode = range.extentNode;
                    if (endNode == null) {
                        endNode = win.document.body;
                    }
                    selection.setBaseAndExtent(startNode, range.startOffset, endNode, range.endOffset);
                }
            }
        }
    },
    getRangeBounding: function(range) {
        if (!range) {
            range = this.getRange();
        }
        if (this._isIE) {
            return {
                left: range.boundingLeft,
                top: range.boundingTop,
                width: range.boundingWidth,
                height: range.boundingHeight
            };
        } else {
            // create a temp element
            var span = this._document.createElement('span');
            span.appendChild(document.createTextNode('a')); // add content or not the offsetHeight is 0
            // insert node at cursor
            rangeClone = range.cloneRange();
            rangeClone.deleteContents();
            rangeClone.insertNode(span);
            // cache useful information
            var bounding = span.getBoundingClientRect();
            // now span is useless, remove it
            span.parentNode.removeChild(span);
            // get
            return {
                left: bounding.left,
                top: bounding.top,
                width: bounding.width,
                height: bounding.height + 2
            };
        }
    },
    cursorAt_IE: function(node, offset) {
        var range = this._document.body.createTextRange();
        if (node) {
            range.moveToElementText(node);
            range.collapse(false);
        } else {
            range.moveToElementText(this._document);
            range.collapse(true);
        }
        if (offset != undefined) {
            range.move('character', offset);
        }
        return range;
    },
    setPoint_FF: function(refRange, node, offset, side) {
        if (node) {
            node = node.nextSibling
        } else {
            node = this._document.body.firstChild;
        }
        if (!node) {
            return false;
        }
        if (offset == 0) {
            refRange['set' + side + 'Before'](node);
            return true;
        }
        var backlog = [];
        var decompose = function(node) {
            if (node.nodeType == 3) {
                backlog.push(node);
            }
            else if ((len = node.childNodes.length) > 0) {
                for (var i = 0; i < len; i++) {
                    decompose(node.childNodes[i]);
                }
            }
        };
        while (true) {
            while (node && !backlog.length) {
                decompose(node);
                node = node.nextSibling;
            }
            var cur = backlog.shift();
            if (!cur) {
                return false;
            }
            var len = cur.nodeValue.length;
            if (len >= offset) {
                refRange['set' + side](cur, offset);
                return true;
            }
            offset -= len;
        }
    },
    selectCursor: function(startNode, startOffset, endNode, endOffset) {
        if (this._isIE) {
            var startRange = this.cursorAt_IE(startNode, startOffset);
            if (startNode != endNode && endNode) {
                var endRange = this.cursorAt_IE(endNode, endOffset);
                startRange.setEndPoint('EndToEnd', endRange);
            }
            this.selectRange(startRange);
        }
        else {
            if (!endNode) {
                endNode = startNode;
                endOffset = startOffset;
            }
            var range = this._document.createRange();
            if (this.setPoint_FF(range, startNode, startOffset, 'End') &&
                this.setPoint_FF(range, endNode, endOffset, 'Start')) {
                this.selectRange(range);
            }
        }
    }
};
//#endregion

//#region
Ext.ux.CurrentLine = function(host) {
    this._host = host;
    this._mirror = host.mirror;
    this._window = host.iframeWin;
    this._document = host.iframeDoc;
    this._selection = new Ext.ux.Selection(this._window);
    // events
    var self = this;
    this._host.AttachEvent(this._document, 'keyup', function(e) {
        self.OnKeyup(new Ext.EventObjectImpl(e));
    });
    this._host.AttachEvent(this._document, 'mouseup', function() {
        self.OnMouseup();
    });
};

Ext.ux.CurrentLine.prototype = {
    // parameters start
    offset: 0,
    number: 1, // start base 1
    endNode: null,
    startNode: null,
    nodeList: [],
    // parameters end

    // events start
    isCursorMovingKey: function(keyCode) {
        // enter, backspace
        if (keyCode == 13 || keyCode == 8) {
            return true;
        }
        // prior, next, end, home, left, up, right, down
        if (keyCode >= 33 && keyCode <= 40) {
            return true;
        }
        return false;
    },
    initialize: function() {
        var setNumber = function() {
            var num = this._mirror.currentLine();
            if (this.number != num) {
                this.number = num;
                // do reset
                this.endNode = null;
                this.startNode = null;
                this.nodeList = [];
            }
        }
        var self = this;
        window.setTimeout(function() {
            setNumber.call(self);
        }, 0);
    },
    OnKeyup: function(e) {
        if (this.isCursorMovingKey(e.getKey())) {
            this.initialize();
        }
    },
    OnMouseup: function() {
        this.initialize();
    },
    // events end

    // selections start
    getRange: function() {
        return this._selection.getRange();
    },
    getBoundingByOffset: function(offset) {
        var node = this.getStartNode();
        if (!node) {
            return null;
        }
        var sel = this._selection;
        if (Ext.isIE) {
            var range = sel.cursorAt_IE(node, offset);
            return sel.getRangeBounding(range);
        } else {
            var refRange = this._document.createRange();
            sel.setPoint_FF(refRange, node, offset, 'End');
            sel.setPoint_FF(refRange, node, offset, 'Start');
            // cache the cursor position offset
            var cursorOffset = this.getCursorOffset();
            // get the range bounding
            var bounding = sel.getRangeBounding(refRange);
            // fix the cursor position
            this.cursorAt(cursorOffset);
            // get
            return bounding;
        }
    },
    // selections end

    getCodeType: function() {
        //TODO:
    },
    getStartNode: function() {
        if (!this.startNode) {
            this.startNode = this._mirror.nthLine(this.number);
            // check
            if (!this.startNode) {
                this.startNode = this._document.body.firstChild;
            }
        }
        return this.startNode;
    },
    getEndNode: function() {
        var startNode = this.getStartNode(), endNode;
        if (startNode) {
            var nodeList = [];
            nodeList.push(startNode);
            while (startNode && (node = startNode.nextSibling)) {
                nodeList.push(node);
                if (node && node.tagName == 'BR') {
                    endNode = node.previousSibling;
                    nodeList.splice(nodeList.length - 1, 1);
                    break;
                }
                startNode = node;
            }
            // set
            this.endNode = endNode;
            this.nodeList = nodeList;
            // check
            if (!this.endNode) {
                this.endNode = this._document.body.lastChild;
            }
        }
        return this.endNode;
    },
    getContainNodes: function() {
        // invoke this method to init the node list
        this.getEndNode();
        return this.nodeList;
    },
    getText: function() {
        var nodes = this.getContainNodes(), result = [];
        for (var i = 0; i < nodes.length; i++) {
            if (node = nodes[i]) {
                result.push(node.innerText || node.textContent || node.nodeValue || '');
            }
        }
        var text = result.join('') || '';
        // clean text
        text = text.replace(/\u00a0/g, ' ').replace(/\u200b/g, '');
        return text;
    },
    setText: function(text) {
        this._mirror.setLineContent(this.getStartNode(), text);
    },
    getCursorOffset: function() {
        this.offset = this._mirror.cursorPosition().character;
        return this.offset;
    },
    cursorAt: function(offset) {
        this.offset = offset;
        this._selection.selectCursor(this.getStartNode(), offset);
    },
    focus: function() {
        this.cursorAt(this.offset);
    }
};
//#endregion

//#region
Ext.ux.CodeMirror = function(config) {
    // enforce defaults
    Ext.apply(config, {
        hideTrigger: true,
        forceSelection: false,
        mode: 'local',
        lazyRender: true,
        typeAhead: false,
        triggerAction: 'all',
        resizable: true,
        syncFont: false,
        listWidth: 150,
        Intelligent: true
    });
    // system api intelligent
    if (store = Ext.ux.getIntelligentDataStore()) {
        config.store = store;
    }
    // default store
    if (!config.store) {
        config.store = new Ext.data.ArrayStore({
            id: 0,
            fields: ['Id', 'Parent', 'Display'],
            systemFields: {
                valueField: 'Id',
                parentField: 'Parent',
                displayField: 'Display'
            },
            data: [
            // js
            [0, '', 'if'],
            [1, '', 'var'],
            [2, '', 'for'],
            [3, '', 'else'],
            [4, '', 'true'],
            [5, '', 'this'],
            [6, '', 'with'],
            [7, '', 'null'],
            [8, '', 'case'],
            [9, '', 'false'],
            [10, '', 'break'],
            [11, '', 'switch'],
            [12, '', 'typeof'],
            [13, '', 'return'],
            [14, '', 'continue'],
            [15, '', 'function'],
            [16, '', 'undefined'],
            [17, '', 'javascript'],
            // document
            [18, '', 'window'],
            [19, '', 'document'],
            [20, 'window', 'document'],
            [21, 'document', 'getElementById'],
            [22, 'document', 'body'],
            // test
            [23, '', 'this'],
            [24, '', 'Controls'],
            [25, 'this', 'Controls'],
            [26, 'Controls', 'Items'],
            [27, 'Controls', 'Clear']
            ]
        });
    }
    if (fields = config.store.systemFields) {
        config.valueField = fields.valueField;
        config.parentField = fields.parentField;
        config.displayField = fields.displayField;
    }
    config.tpl = '<tpl for="."><div class="x-combo-list-item">{' + config.displayField + '}</div></tpl>';
    Ext.ux.CodeMirror.superclass.constructor.call(this, config);
};

Ext.extend(Ext.ux.CodeMirror, Ext.form.ComboBox, {
    activated: false,
    initialized: false,
    disposing: false,
    splitChar: {
        chr: '.',
        charCode: 46,
        keyCode: [110, 190]
    },
    defaultAlignElementHeight: 18,
    adjustSize: Ext.BoxComponent.prototype.adjustSize,
    mirror: null, iframe: null, iframeWin: null, iframeDoc: null, currentLine: null, // important instances
    defaultAutoCreate: {
        tag: 'textarea',
        autocomplete: 'off',
        style: {
            width: 0,
            height: 0,
            display: 'none'
        }
    },
    initComponent: function() {
        Ext.ux.CodeMirror.superclass.initComponent.call(this);
        this.addEvents({
            initialize: true,
            activate: true
        });
    },
    initialize: function() { // fired when iframe loaded
        this.iframeWin = this.iframe.dom.contentWindow;
        this.iframeDoc = Ext.isIE ? this.iframeWin.document : this.iframe.dom.contentDocument;
        this.currentLine = new Ext.ux.CurrentLine(this);
        this.initialized = true;
        this.resize();
        this.initValue();
        this.initEvents();
        this.fireEvent('initialize', this);
    },
    initEvents: function() {
        var self = this, target = (Ext.isIE || Ext.isChrome || Ext.isSafari) ? this.iframeWin : this.iframeDoc;
        this.AttachEvent(target, 'focus', function() {
            self.onFocus();
            self.activated = true;
            self.fireEvent('activate', this);
        });
        this.AttachEvent(target, 'blur', function() {
            self.onBlur();
            self.activated = false;
        });
        this.originalValue = this.getValue();

        // intelligent events, use iframe events replace el
        this.AttachEvent(target, 'scroll', function() { // mousewheel
            self.collapse();
        });
        this.AttachEvent(this.iframeDoc, 'mouseup', function() {
            self.collapse();
        });
        // hold up the iframe method, ensure the event handlers invoked by order
        var self = this;
        var originalMethod = this.mirror.editor.keyDown;
        this.mirror.editor.keyDown = function(ev) {
            var warpedEvent = new Ext.EventObjectImpl(ev);
            // call hold up methods
            self.onKeyUp(warpedEvent); // change use keydown to improve efficiency
            // call original method
            originalMethod.call(self.mirror.editor, ev);
        };
        // custom events
        this.on('expand', this.myExpand, this);
        this.on('collapse', this.myOnCollapse, this);

        //Ext.ux.CodeMirror.superclass.initEvents.call(this);
        //Ext.form.ComboBox.superclass.initEvents.call(this);

        var myCollapse = function(e) {
            this.collapse();
            return true;
        };
        var myCollapseIf = function(e) {
            if (this.isExpanded()) {
                window.setTimeout(function() {
                    if (cursor = self.queryCursor) {
                        var offset = self.currentLine.getCursorOffset();
                        if (offset < cursor.startIndex || offset > cursor.endIndex) {
                            self.collapse();
                        }
                    }
                }, 20);
            }
            return true;
        };
        this.keyNav = new Ext.ux.MyKeyNav(this.iframeDoc, {
            "left": myCollapseIf,
            "right": myCollapseIf,
            "home": myCollapse,
            "end": myCollapse,
            "pageUp": myCollapse,
            "pageDown": myCollapse,
            "del": myCollapse,
            "up": function(e) {
                this.inKeyMode = true;
                this.selectPrev();
            },
            "down": function(e) {
                if (!this.isExpanded()) {
                    //this.onTriggerClick();
                } else {
                    this.inKeyMode = true;
                    this.selectNext();
                }
            },
            "enter": function(e) {
                this.onViewClick();
                this.delayedCheck = true;
                this.unsetDelayCheck.defer(10, this);
            },
            "esc": function(e) {
                this.collapse();
            },
            "tab": function(e) {
                this.onViewClick(false);
                return true;
            },
            doRelay: function(foo, bar, hname) {
                if (hname == 'down1' || this.scope.isExpanded()) {
                    return Ext.KeyNav.prototype.doRelay.apply(this, arguments);
                }
                return true;
            },
            scope: this,
            forceKeyDown: true
        });
        this.queryDelay = Math.max(this.queryDelay || 10,
                this.mode == 'local' ? 10 : 250);
        this.dqTask = new Ext.util.DelayedTask(this.initQuery, this);
        if (this.typeAhead) {
            this.taTask = new Ext.util.DelayedTask(this.onTypeAhead, this);
        }
        if (this.editable !== false && !this.enableKeyEvents) {
            //this.mon(this.el, 'keyup', this.onKeyUp, this);
        }
    },
    initValue: function() {
        if (this.value !== undefined && this.initialized == true) {
            this.setCode(this.value);
        }
    },
    setValue: function(v) {
        this.value = v;
        this.lastSelectionText = v;
        if (this.hiddenField) {
            this.hiddenField.value = v;
        }
        if (this.initialized == true) {
            this.setCode(v);
        }
        return this;
    },
    getValue: function() {
        this.beforeBlur();
        if (this.valueField) {
            return Ext.isDefined(this.value) ? this.value : '';
        } else {
            return Ext.form.ComboBox.superclass.getValue.call(this);
        }
    },
    validateValue: function(value) {
        return true;
    },
    beforeBlur: function() {
        if (this.mirror && this.initialized && this.disposing != true) {
            this.value = this.mirror.getCode();
            this.el.dom.value = this.value;
        }
    },
    onRender: function(ct, position) {
        Ext.ux.CodeMirror.superclass.onRender.call(this, ct, position);
        var area = this.el.dom, self = this, libPath = '/JavaScript/ext/ux/codemirror/',
        options = {
            height: (this.height || 200) + 'px',
            width: (this.width || 300) + 'px',
            content: area.value,
            parserfile: ['parsexml.js', 'parsecss.js', 'tokenizejavascript.js', 'parsejavascript.js', 'parsehtmlmixed.js'],
            stylesheet: [libPath + 'css/xmlcolors.css', libPath + 'css/jscolors.css', libPath + 'css/csscolors.css'],
            path: libPath + 'js/',
            lineNumbers: false,
            initCallback: function() { // iframe load event
                self.initialize.call(self);
            }
        },
        insertFn = function(frame) {
            if (area.nextSibling) {
                area.parentNode.insertBefore(frame, area.nextSibling)
            } else {
                area.parentNode.appendChild(frame);
            }
        };
        this.mirror = new CodeMirror(insertFn, options);
        this.iframe = Ext.get(this.mirror.frame);
        this.iframe.setStyle('border', '#b5b8c8 1px solid');
        this.iframe.setStyle('background-color', '#ffffff');
    },
    afterRender: function() {
        // empty implement, just for override
    },
    initList: function() {
        Ext.ux.CodeMirror.superclass.initList.call(this);
        // ie's bug, when mouse click the resizeer or list scrollbar, focus back to editor
        if (Ext.isIE) {
            if (this.list) {
                this.list.on('mouseup', function() {
                    this.currentLine.focus();
                }, this);
                // mouseup event can not be fired, use scroll event instead but it's no perfect
                this.innerList.on('scroll', this.myFocusIframe, this);
            }
            if (this.resizer) {
                this.resizer.on('resize', this.myFocusIframe, this);
            }
        }
    },
    resize: function() {
        var h = (this.height || this.ownerCt.body.dom.clientHeight) + 'px';
        var w = (this.width || this.ownerCt.body.dom.clientWidth) + 'px';
        this.iframe.setStyle('height', h);
        this.iframe.setStyle('width', w);
    },
    setSize: function(width, height) {
        // support for standard size objects
        if (typeof (width) == 'object') {
            height = width.height;
            width = width.width;
        }
        if (width) {
            this.iframe.setStyle('width', width + 'px');
        }
        if (height) {
            this.iframe.setStyle('height', height + 'px');
        }
        Ext.ux.CodeMirror.superclass.setSize.call(this, width, height);
    },
    onKeyUp: function(e) { // be fire on keydown instead of keyup, in order to override parent method, the method name can not be changed.
        var k = e.getKey();
        // add key to show intelligent by hand
        this.forceQueryByHand = (k == e.F2);
        this.isInputBackspace = (k == e.BACKSPACE);
        this.isInputSplitChar = false;
        if (keyCodes = this.splitChar.keyCode) {
            for (var i = 0, len = keyCodes.length; i < len; i++) {
                if (keyCodes[i] == k) {
                    this.isInputSplitChar = true;
                    break;
                }
            }
        }
        if (this.editable !== false && (this.forceQueryByHand || this.isInputBackspace || !e.isSpecialKey())) {
            this.lastKey = k;
            this.dqTask.delay(this.queryDelay); // do initQuery method
        }
        Ext.form.ComboBox.superclass.onKeyUp.call(this, e);
    },
    initQuery: function() {
        // first line has some bug, so except it
        if (this.currentLine.number == 1) {
            return;
        }
        // do format automatic
        if (this.isInputSplitChar && this.isExpanded()) {
            this.myFormatParentField();
        }
        // get query value
        var value = this.myGetQueryValue();
        if (value == this.splitChar.chr) {
            value = null;
        }
        if (value || this.forceQueryByHand) {
            this.doQuery(value);
            // fix a ff problem, this is the last point of once query
            if (this.forceQueryByHand && !Ext.isIE) {
                this.myFocusIframe();
            }
        } else {
            this.collapse();
        }
    },
    doQuery: function(q, forceAll) {
        q = Ext.isEmpty(q) ? '' : q;
        var qe = {
            query: q,
            forceAll: forceAll,
            combo: this,
            cancel: false
        };
        if (this.fireEvent('beforequery', qe) === false || qe.cancel) {
            return false;
        }
        q = qe.query;
        forceAll = qe.forceAll;
        if (forceAll === true || (q.length >= this.minChars)) {
            if (this.lastQuery !== q || this.forceQueryByHand) {
                this.prevQuery = this.lastQuery || ''; // crl
                this.lastQuery = q;
                if (this.mode == 'local') {
                    this.selectedIndex = -1;
                    if (forceAll) {
                        this.store.clearFilter();
                    } else {
                        this.store.filterBy(this.myFilterMethod, this); // crl
                    }
                    this.onLoad();
                } else {
                    this.store.baseParams[this.queryParam] = q;
                    this.store.load({
                        params: this.getParams(q)
                    });
                    this.expand();
                }
            } else {
                this.selectedIndex = -1;
                this.onLoad();
            }
        }
    },
    onViewClick: function(doFocus) {
        if (doFocus !== false) {
            this.myFocusIframe();
        }
        var index = this.view.getSelectedIndexes()[0];
        var r = this.store.getAt(index);
        if (r) {
            this.onSelect(r, index);
        }
    },
    onSelect: function(record, index) {
        if (this.fireEvent('beforeselect', this, record, index) !== false) {
            var text = record.data[this.displayField];
            var parentText = record.data[this.parentField];
            if (parentText) {
                text = parentText + this.splitChar.chr + text;
            }
            this.myInsertIntelligent(text);
            this.collapse();
            this.fireEvent('select', this, record, index);
        }
    },
    restrictHeight: function() {
        this.innerList.dom.style.height = '';
        var inner = this.innerList.dom;
        var pad = this.list.getFrameWidth('tb') + (this.resizable ? this.handleHeight : 0) + this.assetHeight;
        var h = Math.max(inner.clientHeight, inner.offsetHeight, inner.scrollHeight);
        var ha = this.getPosition()[1] - Ext.getBody().getScroll().top;
        var hb = Ext.lib.Dom.getViewHeight() - ha - this.myGetAlignElementHeight();
        var space = Math.max(ha, hb, this.minHeight || 0) - this.list.shadowOffset - pad - 5;
        h = Math.min(h, space, this.maxHeight);

        this.innerList.setHeight(h);
        this.list.beginUpdate();
        this.list.setHeight(h + pad);
        this.list.alignTo(this.myGetAlignElement(), this.listAlign); // crl 
        this.list.endUpdate();
    },
    expand: function() {
        if (this.isExpanded() || !this.hasFocus) {
            return;
        }
        this.list.alignTo(this.myGetAlignElement(), this.listAlign); // crl
        this.list.show();
        if (Ext.isGecko2) {
            this.innerList.setOverflow('auto');
        }
        Ext.getDoc().on({
            scope: this,
            mousewheel: this.collapseIf,
            mousedown: this.collapseIf
        });
        this.fireEvent('expand', this);
    },
    collapseIf: function(e) {
        if (!e.within(this.list)) {
            this.collapse();
        }
    },
    getCode: function() {
        if (this.mirror && this.initialized) {
            return this.mirror.getCode();
        } else {
            return this.el.dom.value;
        }
    },
    setCode: function(code) {
        this.el.dom.value = code;
        if (this.mirror && this.initialized) {
            this.mirror.setCode(code);
        }
    },
    destroy: function() {
        this.disposing = true;
        this.ClearEvents();
        this.iframe.dom.src = 'javascript:false;';
        window.clearInterval(CodeMirror.sizeBarIntervalId);
        Ext.removeNode(this.iframe.dom);
        this.iframe.remove();
        if (this.alignElement) {
            this.alignElement.remove();
        }
        Ext.ux.CodeMirror.superclass.destroy.call(this);
    },

    // events management for iframe start
    Events: [],
    AttachEvent: function(obj, type, fn) {
        if (obj.addEventListener) {
            obj.addEventListener(type, fn, false);
        }
        if (obj.attachEvent) {
            obj.attachEvent('on' + type, fn);
        }
        this.Events.push({ obj: obj, type: type, fn: fn });
    },
    DetachEvent: function(obj, type, fn) {
        try {
            if (obj.removeEventListener) {
                obj.removeEventListener(type, fn);
            }
            if (obj.detachEvent) {
                obj.detachEvent('on' + type, fn);
            }
        }
        catch (ee) { }
    },
    ClearEvents: function() {
        for (var i = 0; i < this.Events.length; i++) {
            with (this.Events[i]) {
                this.DetachEvent(obj, type, fn);
            }
        }
        this.Events = [];
    },
    // events management for iframe end

    // my methods start
    myGetListBounding: function() {
        var getNew = false;
        if (this.isInputBackspace) {
            if (this.prevQuery.indexOf(this.lastQuery) == -1 ||
                this.prevQuery.replace(this.lastQuery, '') == this.splitChar.chr
                ) {
                getNew = true;
            }
        } else if (this.isInputSplitChar) {
            getNew = true;
        }
        if (this.isExpanded() && !getNew) {
            return null;
        }
        if (cursor = this.queryCursor) {
            var offset = cursor.startIndex;
            if ((splitIndex = this.lastQuery.indexOf(this.splitChar.chr)) != -1) {
                offset += splitIndex + 1;
            }
            var frameXY = this.iframe.getXY();
            var bounding = this.currentLine.getBoundingByOffset(offset);
            bounding.left += frameXY[0];
            bounding.top += frameXY[1];
            return bounding;
        }
    },
    myExpand: function() {
        // a flag use for prevent the iframe's enter event
        window.codeIntelligenting = true;
    },
    myOnCollapse: function() {
        this.queryCursor = null;
        // timeout to reset the flag when collapse
        window.setTimeout(function() {
            window.codeIntelligenting = false;
        }, 20);
    },
    myFocusIframe: function() {
        //this.mirror.focus();
        if (this.iframeWin) {
            this.iframeWin.focus();
        }
    },
    myValidChar: function(chr) {
        var charCode = chr.charCodeAt(0);
        if (charCode >= 97 && charCode <= 122) { // "a-z"
            return true;
        }
        if (charCode >= 65 && charCode <= 90) { // "A-Z"
            return true
        }
        if (charCode >= 48 && charCode <= 57) { // "0-9"
            return true;
        }
        if (charCode == 95) { // "_"
            return true;
        }
        if (charCode == this.splitChar.charCode) { // "."
            return true;
        }
        return false;
    },
    myGetQueryValue: function() {
        var offset = this.currentLine.getCursorOffset();
        var lineText = this.currentLine.getText();
        var point = offset - 1, len = lineText.length;
        if (len >= point) {
            var result = [], chr, back, forward, splitNum = 0,
                splitChar = this.splitChar.chr;
            // search back
            for (back = point; back > -1; back--) {
                chr = lineText.charAt(back);
                if (chr == splitChar) {
                    splitNum++;
                    if (splitNum == 2) {
                        break;
                    }
                    result.unshift(chr);
                    continue;
                }
                //if (/^[A-Za-z0-9]+$/.test(chr)) {
                if (this.myValidChar(chr)) {
                    result.unshift(chr);
                } else {
                    break;
                }
            }
            // search forward
            for (forward = point + 1; forward < len; forward++) {
                chr = lineText.charAt(forward);
                if (chr == splitChar) {
                    break;
                }
                //if (/^[A-Za-z0-9]+$/.test(chr)) {
                if (this.myValidChar(chr)) {
                    result.push(chr);
                } else {
                    break;
                }
            }
            // cache the query information
            this.queryCursor = {
                lineText: lineText,
                startIndex: back + 1,
                endIndex: forward
            };
            // get
            return result.join('');
        }
    },
    myFilterMethod: function(record, id) {
        var data = record.data,
        query = this.lastQuery.toLowerCase(),
        parentText = data[this.parentField].toLowerCase(),
        displayText = data[this.displayField].toLowerCase(),
        index = query.lastIndexOf(this.splitChar.chr);
        if (index == -1 && parentText == '') {
            return displayText.indexOf(query) == 0;
        } else {
            var temp = query.substr(index + 1);
            return (parentText == query.substring(0, index)) &&
                   (temp == '' || displayText.indexOf(temp) == 0);
        }
    },
    myInsertIntelligent: function(value) {
        if (!this.isExpanded() || Ext.isEmpty(value)) {
            return;
        }
        // get the query information cached
        if (cursor = this.queryCursor) {
            // get line text
            var startText = cursor.lineText.substring(0, cursor.startIndex),
            endText = cursor.lineText.substr(cursor.endIndex),
            lineText = startText + value,
            offset = lineText.length;
            lineText += endText;
            // set line text
            this.currentLine.setText(lineText);
            // fix cursor position
            this.currentLine.cursorAt(offset);
        }
    },
    myFormatParentField: function(e) {
        // get the selected data
        var index = this.view.getSelectedIndexes()[0];
        var record = this.store.getAt(index);
        if (record) {
            // get insert text
            var text = record.data[this.displayField];
            var parentText = record.data[this.parentField];
            if (parentText) {
                text = parentText + this.splitChar.chr + text;
            }
            if (cursor = this.queryCursor) {
                // judge if different
                if (cursor.lineText.substr(cursor.startIndex) != text) {
                    this.myInsertIntelligent(text + this.splitChar.chr);
                }
            }
        }
    },
    myGetAlignElement: function() {
        var bounding = this.myGetListBounding();
        if (!this.alignElement) {
            var height = this.defaultAlignElementHeight;
            if (bounding && bounding.height) {
                height = bounding.height;
            }
            var div = Ext.get(document.createElement('div'));
            div.setStyle({
                width: '0px',
                height: height + 'px',
                position: 'absolute'
            });
            Ext.getBody().appendChild(div);
            this.alignElement = div;
        }
        if (bounding) {
            this.alignElement.setXY([bounding.left, bounding.top]);
        }
        return this.alignElement;
    },
    myGetAlignElementHeight: function() {
        if (this.alignElement) {
            return this.alignElement.getHeight();
        } else {
            return this.defaultAlignElementHeight;
        }
    }
    // my methods end
});

Ext.ComponentMgr.registerType('codemirror', Ext.ux.CodeMirror); // register
//#endregion

// query order:
// onKeyup -> initQuery -> myFormatParentField -> myGetQueryValue -> doQuery -> myFilterMethod -> myFocusIframe